<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/


class Storm_Model_PersistenceStrategy_Db extends Storm_Model_PersistenceStrategy_Abstract {

  protected $_table;

  public function newFromRow($row) {
    return $this->_loader->newFromRow($row);
  }


  public function getTable() {
    if (!isset($this->_table)) {
      $table_name = call_user_func_array([$this->_loader->getModel(),'getClassVar'],['_table_name']);
      $this->_table = new Storm_Model_Table(array('name' => $table_name));
    }

    return $this->_table;
  }


  public function setTable($tbl) {
    $this->_table = $tbl;
    return $this;
  }


  public function find($id) {
    $rowset = $this->getTable()->find($id)->toArray();

    if (count($rowset) > 0) {
      $instance = $this->newFromRow($rowset[0]);
      return $instance;
    }

    return null;
  }

  public function update($data,$id) {
    return $this->getTable()
      ->update($data, $this->_loader->getIdField(). "='" . $id . "'");

  }


  public function delete($model) {
    return $this->getTable()
                ->delete($this->_loader->getIdField() . "='" . $model->getId() . "'");
  }


  public function findAll($select=null) {
    if (is_string($select))
      return $this->_findAllByString($select);

    return ($rowset = $this->getTable()->fetchAll($select))
      ? array_map([$this->_loader, 'newFromRow'], $rowset->toArray())
      : [];
  }


  protected function _findAllByString($query) {
    return array_map([$this->_loader, 'newFromRow'],
                     $this->getTable()->getAdapter()->fetchAll($query));
  }


  public function findAllBy($args) {
    if ($select = $this->_generateSelectFor($args))
      return $this->findAll($select);
    return [];
  }


  public function fetchAllBy($fields, $params) {
    if (!$select = $this->_generateSelectFor($params))
      return [];

    $select->from($select->getTable(), $fields);
    return ($rowset = $this->getTable()->fetchAll($select))
      ? $rowset->toArray()
      : [];
  }


  public function countBy($args) {
    if (!$select = $this->_generateSelectFor($args))
      return 0;

    $select->from($this->getTable(),
                  [sprintf('count(*) as numberof')]);
    $rows = $this->getTable()->fetchAll($select)->toArray();
    return $rows[0]['numberof'];
  }

  public function insert($data) {
    return $this->getTable()->insert($data);
  }

  public function lastInsertId() {
    return $this->getTable()->getAdapter()->lastInsertId();
  }

  /**
   * @see findAllBy
   */
  public function _generateSelectFor($args) {
    if (array_key_exists('role', $args) && array_key_exists('model', $args)) {
      $model = $args['model'];
      $role = $args['role'];
      unset($args['model']);
      unset($args['role']);

      if ($model->isNew()) return null;

      $select = $this->getTable()->select();
      $field = $this->_loader->getIdFieldForDependent($role);
      $select = $select->where($field . '=?', $model->getId());
    }
    else
      $select = $this->getTable()->select();

    foreach ($args as $field => $value) {
      if (in_array($field, ['order', 'limit', 'where'])) {
        $select->$field($value);
        continue;
      }

      if (in_array($field, ['limitPage', 'scope'])) {
        $this->$field($select, $value);
        continue;
      }

      $negated = (' not' == substr($field, -4));
      $field = $negated ? substr($field, 0, strlen($field) - 4): $field;

      if (is_array($value)) {
        $select->where($field . ($negated ? ' not': '') . ' in (?)', $value);
        continue;
      }

      if (null === $value) {
        $select->where($field . ' is' . ($negated ? ' not' : '') . ' null');
        continue;
      }

      $select->where($field . ($negated ? '!': '') . '=?', $value);
    }

    return $select;
  }


  protected function limitPage($select, $value) {
    if (is_array($value) && 2 <= count($value))
      $select->limitPage($value[0], $value[1]);
  }


  protected function scope($select, $value) {
    foreach($value as $scope_field => $scope_value)
      $select->where($scope_field . '=?', $scope_value);
  }
}
?>