<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

class Storm_Test_XPath {
  public static function newInstance() {
    return new self();
  }
  
  public function assertQueryContentContains($xml, $path, $match, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__, $match)) {
            $constraint->fail($path, $message);
    }
  }


  public function assertNotQueryContentContains($xml, $path, $match, $message = '') {
    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__, $match)) {
            $constraint->fail($path, $message);
    }
  }


  public function assertXpath($xml, $path, $message = '') {
    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__)) {
            $constraint->fail($path, $message);
    }
  }


  public function assertXpathCount($xml, $path, $count, $message = '') {
    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__, $count)) {
            $constraint->fail($path, $message);
    }
  }


  public function assertNotXpath($xml, $path, $message = '') {
    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__)) {
            $constraint->fail($path, $message);
    }
  }


  public function assertXpathContentContains($xml, $path, $match, $message = '') {
    if (!$message)
      $message = sprintf("Document:\n\n%s\n", $xml);

    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__, $match)) {
      $constraint->fail($path, $message);
    }
  }


  public function assertNotXpathContentContains($xml, $path, $match, $message = '') {
    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__, $match)) {
      $constraint->fail($path, $message);
    }
  }


  public function assertQueryCount($xml, $path, $count, $message = '')
  {
    $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
    if (!$constraint->evaluate($xml, __FUNCTION__, $count)) {
      $constraint->fail($path, $message);
    }
  }


  /** 
   * @param string $xml
   * @param array $paths
   * @param string $message
   */
  public function assertAnyXpath($xml, array $paths, $message = '') {
    foreach ($paths as $path) {
      $constraint = new Zend_Test_PHPUnit_Constraint_DomQuery($path);
      if ($constraint->evaluate($xml, 'assertXpath')) {
        return;
      }
    }

    $this->fail("Failed asserting any path from (\n\t" . implode(", \n\t", $paths)  
                . ') EXISTS' . "\n" . $message);
  }
}