<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2020 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/


abstract class Storm_Model_EventTriggeringTestCase extends Storm_Test_ModelTestCase {
  protected
    $_events,
    $_callable,
    $_called = false;

  public function setUp() {
    parent::setUp();
    Storm_Model_Loader::defaultToVolatile();
    $this->_events = Storm_Events::getInstance();
  }


  public function tearDown() {
    Storm_Model_Loader::defaultToDb();
    parent::tearDown();
  }


  /** @test */
  public function withRegisteredClosureShouldCallBackOnSave() {
    $this->_events->register($this->_callable);
    (new Storm_Model_EventTriggeringModel)->save();
    $this->assertTrue($this->_called);
  }


  /** @test */
  public function withUnregisteredClosureShouldNotCallbackOnSave() {
    $this->_events->register($this->_callable)
                  ->unregister($this->_callable);
    (new Storm_Model_EventTriggeringModel)->save();

    $this->assertFalse($this->_called);
  }
}




class Storm_Model_EventTriggeringClosureTest extends Storm_Model_EventTriggeringTestCase {
  public function setUp() {
    parent::setUp();
    $this->_callable = function($event) { $this->_called = true; };
  }
}




class Storm_Model_EventTriggeringCallableArrayTest extends Storm_Model_EventTriggeringTestCase {
  public function setUp() {
    parent::setUp();
    $this->_callable = [$this, 'onSave'];
  }


  public function onSave($event) {
    $this->_called = true;
  }
}




class Storm_Model_EventTriggeringInvokableTest extends Storm_Model_EventTriggeringTestCase {
  public function setUp() {
    parent::setUp();
    $this->_callable = new Storm_Model_EventTriggeringInvokable($this);
  }


  public function beCalled() {
    $this->_called = true;
  }


  /** @test */
  public function withUnregisteredByClassNameShouldNotCallbackOnSave() {
    $this->_events->register($this->_callable)
                  ->unregisterClass(Storm_Model_EventTriggeringInvokable::class);
    (new Storm_Model_EventTriggeringModel)->save();

    $this->assertFalse($this->_called);
  }
}




class Storm_Model_EventTriggeringModel extends Storm_Model_Abstract {}




class Storm_Model_EventTriggeringInvokable {
  protected $_test;

  public function __construct($test) {
    $this->_test = $test;
  }


  public function __invoke($event) {
    $this->_test->beCalled();
  }
}