<?php
/*
STORM is under the MIT License (MIT)

Copyright (c) 2010-2011 Agence Française Informatique http://www.afi-sa.fr

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

class Storm_Test_SaveHooksTest extends Storm_Test_ModelTestCase {
  public function setUp() {
    $this->trace_hook_mock = $this->mock();

    $this->valid_user = new Storm_Test_Mock_User();
    $this->valid_user
      ->setValid(true)
      ->setTraceHookMock($this->trace_hook_mock);

    $this->invalid_user = new Storm_Test_Mock_User();
    $this->invalid_user
      ->setValid(false)
      ->setTraceHookMock($this->trace_hook_mock);
  }


  public function testInvalidModelDoNotCallHooks() {
    $this->trace_hook_mock
      ->whenCalled('beforeSave')
      ->never()
      ->whenCalled('afterSave')
      ->never()
      ->beStrict();

    $this->invalid_user->save();
  }


  public function testValidModelCallHooksWithSuccessfullSave() {
    $this->trace_hook_mock
      ->whenCalled('beforeSave')
      ->willDo(function()
               {
                 $this->trace_hook_mock
                   ->whenCalled('loaderSave')
                   ->willDo(function()
                            {

                              $this->trace_hook_mock
                                ->whenCalled('afterSave')
                                ->answers(true);
                              return 2;
                            });

               })
      ->beStrict();


    $this->valid_user->save();
    $this->assertTrue($this->trace_hook_mock->methodHasBeenCalled('afterSave'));
  }


  public function testValidModelCallHooksWithUnsuccessfullSave() {
    $this->trace_hook_mock
      ->whenCalled('beforeSave')
      ->willDo(function()
               {
                 $this->trace_hook_mock
                   ->whenCalled('loaderSave')
                   ->willDo(function()
                            {

                              $this->trace_hook_mock
                                ->whenCalled('afterSave')
                                ->answers(true);
                              return false;
                            });

               })
      ->beStrict();


    $this->valid_user->save();
    $this->assertTrue($this->trace_hook_mock->methodHasBeenCalled('afterSave'));

  }
}

?>